@@ -123,8 +123,10 @@ class NumbersModel:
         #     },
         #
         #
-        ce_id = self.find_refs("CalculationEngineArchive")[0]
-        calc_engine = self.objects[ce_id]
+        calc_engine = self.calc_engine()
+        if calc_engine is None:
+            return {}
+
         owner_id_map = {}
         for e in calc_engine.dependency_tracker.owner_id_map.map_entry:
             owner_id_map[e.internal_owner_id] = uuid(e.owner_id)
@@ -189,10 +191,13 @@ class NumbersModel:
         #         }
         #     ]
         # }
-        cell_records = []
+        calc_engine = self.calc_engine()
+        if calc_engine is None:
+            return []
+
         table_base_id = self.table_base_id(table_id)
-        ce_id = self.find_refs("CalculationEngineArchive")[0]
-        for finfo in self.objects[ce_id].dependency_tracker.formula_owner_info:
+        cell_records = []
+        for finfo in calc_engine.dependency_tracker.formula_owner_info:
             if finfo.HasField("cell_dependencies"):
                 formula_owner_id = uuid(finfo.formula_owner_id)
                 if formula_owner_id == table_base_id:
@@ -201,6 +206,14 @@ class NumbersModel:
                             cell_records.append((cell_record.row, cell_record.column))
         return cell_records
 
+    @lru_cache(maxsize=None)
+    def calc_engine(self):
+        ce_id = self.find_refs("CalculationEngineArchive")
+        if len(ce_id) == 0:
+            return None
+        else:
+            return self.objects[ce_id[0]]
+
     @lru_cache(maxsize=None)
     def merge_cell_ranges(self, table_id):
         """Exract all the merge cell ranges for the Table."""
@@ -467,22 +480,13 @@ class NumbersModel:
     def table_cell_formula_decode(
         self, table_id: int, row_num: int, col_num: int, cell_type: int
     ):
-        storage_buffer = self.storage_buffer(table_id, row_num, col_num)
         if not self.table_formulas(table_id).is_formula(row_num, col_num):
             return None
 
-        # TODO: Why is this required? Which cell types are affected?
-        mod_offset = unpack("<h", storage_buffer[6:8])[0] == 1
-
-        if cell_type == TSTArchives.formulaErrorCellType:
-            formula_key = unpack("<h", storage_buffer[-4:-2])[0]
-        elif (
-            cell_type == TSTArchives.textCellType
-            or cell_type == TSTArchives.durationCellType
-        ) and mod_offset:
-            formula_key = unpack("<h", storage_buffer[-16:-14])[0]
-        else:
-            formula_key = unpack("<h", storage_buffer[-12:-10])[0]
+        buffer = self.storage_buffer_pre_bnc(table_id, row_num, col_num)
+        flags = unpack("<i", buffer[4:8])[0]
+        data_offset = 8 + bin(flags & 0x0D8E).count("1") * 4
+        formula_key = unpack("<h", buffer[data_offset : data_offset + 2])[0]
 
         return formula_key
 
@@ -591,7 +595,7 @@ def uuid(ref: dict) -> int:
 
 
 def storage_buffer_value(buffer, cell_type):
-    """Decode data values from a sturage buffer based on the type of data represented"""
+    """Decode data values from a storage buffer based on the type of data represented"""
     cell_value = CellValue(cell_type)
     flags = unpack("<i", buffer[4:8])[0]
     data_offset = 12 + bin(flags & 0x0D8E).count("1") * 4
